@@ -1,5 +1,5 @@
 from tornado import web, httpclient, ioloop, httputil
-import os, json
+import os, json, re
 import aiohttp, urllib
 import socket
 import subprocess
@@ -41,6 +41,7 @@ class ProxyHandler(HubOAuthenticated, WebSocketHandlerMixin):
         self.subprotocols = None
         self.forward_user_info = kwargs.pop('forward_user_info', False)
         self.query_user_info = kwargs.pop('query_user_info', False)
+        self.progressive = kwargs.pop('progressive', False)
         super().__init__(*args, **kwargs)
 
     @property
@@ -151,7 +152,7 @@ class ProxyHandler(HubOAuthenticated, WebSocketHandlerMixin):
 
         return client_uri
 
-    def _build_proxy_request(self, host, port, proxied_path, body):
+    def _build_proxy_request(self, host, port, proxied_path, body, **extra_opts):
 
         headers, query_args = self._filter_headers_and_query(self.proxy_request_headers(), self.proxy_query_arguments())
 
@@ -165,7 +166,7 @@ class ProxyHandler(HubOAuthenticated, WebSocketHandlerMixin):
 
         req = httpclient.HTTPRequest(
             client_uri, method=self.request.method, body=body,
-            headers=headers, **self.proxy_request_options())
+            headers=headers, **self.proxy_request_options(), **extra_opts)
         return req
 
     def _filter_headers_and_query(self, headers, query_args):
@@ -192,6 +193,8 @@ class ProxyHandler(HubOAuthenticated, WebSocketHandlerMixin):
             # Take internal _hub_auth_user_cache property of jupyterhub.services.auth.HubAuthenticated
             # Only include headline info in case, e.g. secret auth info is stored on the user object
 
+            self.log.info(self._hub_auth_user_cache)
+
             user_info_str = json.dumps(dict(
                     [(k, self._hub_auth_user_cache.get(k, None)) for k in ('kind', 'name', 'admin', 'groups')]
                 ))
@@ -243,6 +246,82 @@ class ProxyHandler(HubOAuthenticated, WebSocketHandlerMixin):
             else:
                 body = None
 
+        if self.progressive:
+            return await self._proxy_progressive(host, port, proxied_path, body)
+        else:
+            return await self._proxy_normal(host, port, proxied_path, body)
+
+    async def _proxy_progressive(self, host, port, proxied_path, body):
+        # Proxy in progressive flush mode, whenever chunks are received. Potentially slower but get results quicker for voila
+
+        client = httpclient.AsyncHTTPClient()
+
+        # Set up handlers so we can progressively flush result
+
+        headers_raw = []
+
+        def dump_headers(headers_raw):
+            for line in headers_raw:
+                r = re.match('^([a-zA-Z0-9\-_]+)\s*\:\s*([^\r\n]+)[\r\n]*$', line)
+                if r:
+                    k,v = r.groups([1,2])
+                    if k not in ('Content-Length', 'Transfer-Encoding',
+                                  'Content-Encoding', 'Connection'):
+                        # some header appear multiple times, eg 'Set-Cookie'
+                        self.set_header(k,v)
+                else:
+                    r = re.match('^HTTP[^\s]* ([0-9]+)', line)
+                    if r:
+                        status_code = r.group(1)
+                        self.set_status(int(status_code))
+            headers_raw.clear()
+
+        # clear tornado default header
+        self._headers = httputil.HTTPHeaders()
+
+        def header_callback(line):
+            headers_raw.append(line)
+
+        def streaming_callback(chunk):
+            # Do this here, not in header_callback so we can be sure headers are out of the way first
+            dump_headers(headers_raw) # array will be empty if this was already called before
+            self.write(chunk)
+            self.flush()
+
+        # Now make the request
+
+        req = self._build_proxy_request(host, port, proxied_path, body, 
+                    streaming_callback=streaming_callback, 
+                    header_callback=header_callback)
+
+        try:
+            response = await client.fetch(req, raise_error=False)
+        except httpclient.HTTPError as err:
+            if err.code == 599:
+                self._record_activity()
+                self.set_status(599)
+                self.write(str(err))
+                return
+            else:
+                raise
+
+        # record activity at start and end of requests
+        self._record_activity()
+
+        # For all non http errors...
+        if response.error and type(response.error) is not httpclient.HTTPError:
+            self.set_status(500)
+            self.write(str(response.error))
+        else:
+            self.set_status(response.code, response.reason) # Should already have been set
+
+            dump_headers(headers_raw) # Should already have been emptied
+
+            if response.body: # Likewise, should already be chunked out and flushed
+                self.write(response.body)
+
+    async def _proxy_normal(self, host, port, proxied_path, body):
+        # Proxy using normal mode - only write on completion
         client = httpclient.AsyncHTTPClient()
 
         req = self._build_proxy_request(host, port, proxied_path, body)
